#### SIMPLE SIMULATION, COMMON FACTOR MODEL
setwd(".")
rm(list = ls(all = TRUE)) #Clear workspace


n.iterate <- 10000 #number of iterations
cont  <- 0         #count indicator
gdp.data <- c(t(read.table("rgdpl1980.txt", header = TRUE))) #used for the individual effects and the covariates


for(J   in c(5,20)){   
for(T0  in c(5,20)){  #simulation 1 is carried out for J=5,20 and T0=5,20.

    cont  <- cont + 1
    T1 <- T0  #Pre and post treatment periods, T0 and T1 respectively. 
    
    T.total <- T0+T1
    time.pretr <- 1:T0
    time.tr <- (T0+1):T.total
    time.var <- seq(from=1, to = (T.total), by=1) 
    no.controls <- J
    
    
    for (j in 1:n.iterate){
      
      rs <- sample(1:157,2, replace=TRUE)  
      initial_values <- gdp.data[rs]
      
      #common trend 1 tau_t = mu + tau_t-1 + epsilon_t 
      mu <- rchisq(1,1)
      tau_0 <- initial_values[1]
      epsilon <- rnorm(T0+T1,0,0.5)
      ctrend <- vector() 
      ctrend[1] <- mu + tau_0 + epsilon[1]
      for (i in 2:(T0+T1)){
        ctrend[i] <- mu + ctrend[i-1] + epsilon[i]
      }
      
      #common trend 2 tau_t = mu + tau_t-1 + epsilon_t 
      mu <- rchisq(1,1)
      tau_0 <- initial_values[2]
      epsilon <- rnorm(T0+T1,0,0.5)
      ctrend2 <- vector() 
      ctrend2[1] <- mu + tau_0 + epsilon[1]
      for (i in 2:(T0+T1)){
        ctrend2[i] <- mu + ctrend2[i-1] + epsilon[i]
      }
      
      #cyclical component 1
      rho <- 0.8
      c_0 <- 0
      v <- rnorm(T0+T1,0,0.5)
      cicle <- vector() 
      cicle[1] <- rho*c_0 + v[1]
      for (i in 2:(T0+T1)){
        cicle[i] <- rho*cicle[i-1] + v[i]
      }
      
      #cyclical component 2
      rho <- 0.8
      c_0 <- 0
      v <- rnorm(T0+T1,0,0.5)
      cicle2 <- vector() 
      cicle2[1] <- rho*c_0 + v[1]
      for (i in 2:(T0+T1)){
        cicle2[i] <- rho*cicle2[i-1] + v[i]
      }
      
      #factor loading matrix
      factor.load <- matrix(NA, nrow=(J+1), ncol=4)
      factor.load[,1] <- rchisq(nrow(factor.load), 1)
      factor.load[,2] <- rchisq(nrow(factor.load), 1)
      factor.load[,3] <- rnorm(nrow(factor.load), 1, 1)
      factor.load[,4] <- rnorm(nrow(factor.load), 1, 1)
      factor.load.norm <- matrix(NA, nrow=(J+1), ncol=4)
      #normalized
      for (k in 1:nrow(factor.load)){
        factor.load.norm[k,] <- factor.load[k,]/sum(factor.load[k,])
      }

      #covariate (theta_tz_j)
      covariate <- sample(1:157,J+1, replace=TRUE)
      z <- matrix(gdp.data[covariate], nrow=(J+1), ncol=1)
      theta_t <- seq(0.3,0.9, length.out = T0+T1)
      #to weight the factor and indiv effect (aprox.) equally we normalize
      #z as well
      aggcovar <- z%*%theta_t
      aggcovar <- (aggcovar-mean(aggcovar))/sd(aggcovar)
      
      #indiv effect (gamma)
      indiv_effect <- sample(1:157,J+1, replace=TRUE)
      gamma <- matrix(gdp.data[indiv_effect], nrow=(J+1), ncol=1)
      #to weight the factor and indiv effect (aprox.) equally we normalize
      #gamma as well
      gamma.norm <- gamma%*%rep(1,T0+T1)
      gamma.norm <- (gamma.norm-mean(gamma.norm))/sd(gamma.norm)
      
      
      #error
      error <- matrix(rnorm((T0+T1)*(J+1),0,0.25), nrow=(J+1), ncol=(T0+T1))
      
      #outcome, Y, depends on the simulation.
      #Simulation #1: we have four common factors:
      Y_sim1 <- (factor.load.norm[,1]%*%t(ctrend) + factor.load.norm[,2]%*%t(ctrend2)+
                 factor.load.norm[,3]%*%t(cicle)  + factor.load.norm[,4]%*%t(cicle2) + error)
      
      #Simulation #2: single common (cicle) factor:
      Y_sim2a <- (factor.load.norm[,3]%*%t(cicle) + error)
      
      #Simulation #2: single common (cicle) factor and a covariate:
      Y_sim2b <- (factor.load.norm[,3]%*%t(cicle) + aggcovar + error)
      
      #Simulation #2: single common (cicle) factor and an individual effect:
      Y_sim2c <- (factor.load.norm[,3]%*%t(cicle) + gamma.norm + error)
      
      #Simulation #2: single common (cicle) factor, covariate and an individual effect:
      Y_sim2d <- (factor.load.norm[,3]%*%t(cicle) + aggcovar + gamma.norm + error)
      
      #This gives use the outcome variable for the treated unit and J covariates for each iteration
      treated.data <- t(Y)
      
      #Then the synthetic control and the panel data is applied for each iteration and the results stored.
      
    
    } #end iterations loop
}} #end J, T0 cases loops